import ftvstt.providers.vocapia
import ftvstt.providers.bertin
import ftvstt.providers.google
import ftvstt.providers.amazon
import ftvstt.transcripts

Vocapia = ftvstt.providers.vocapia.Vocapia
Bertin = ftvstt.providers.bertin.Bertin
Google = ftvstt.providers.google.Google
Amazon = ftvstt.providers.amazon.Amazon
Transcript = ftvstt.transcripts.Transcript
